@@ -3,6 +3,7 @@ import random
 import numpy as np
 from collections import Counter
 import warnings
+import ctypes
 
 from qlazy.config import *
 from qlazy.error import *
@@ -11,8 +12,6 @@ from qlazy.Observable import *
 from qlazy.PauliProduct import *
 from qlazy.lib.qstate_mcx import *
 
-MDATA_TABLE = {}
-
 class QState(ctypes.Structure):
     """ Quantum State
 
@@ -37,7 +36,7 @@ class QState(ctypes.Structure):
         ('gbank', ctypes.c_void_p),
     ]
 
-    def __new__(cls, qubit_num=None, vector=None, seed=None):
+    def __new__(cls, qubit_num=None, vector=None, seed=None, **kwargs):
         """
         Parameters
         ----------
@@ -61,10 +60,13 @@ class QState(ctypes.Structure):
                 print("qubit number must be {0:d} or less.".format(MAX_QUBIT_NUM))
                 raise QState_Error_Initialize()
 
-            return qstate_init(qubit_num, seed)
+            obj = qstate_init(qubit_num, seed)
 
         else:
-            return qstate_init_with_vector(vector, seed)
+            obj = qstate_init_with_vector(vector, seed)
+            
+        self = ctypes.cast(obj.value, ctypes.POINTER(cls)).contents
+        return self
 
     def __str__(self):
 
@@ -79,6 +81,11 @@ class QState(ctypes.Structure):
         qid : list, default - qubit id's list for all of the qubits
             qubit id's list to reset.
 
+        Returns
+        -------
+        self : instance of QState
+            quantum state.
+
         Notes
         -----
         If 'qid' is set, specified qubits are reset after
@@ -88,6 +95,7 @@ class QState(ctypes.Structure):
 
         """
         qstate_reset(self, qid=qid)
+        return self
         
     @classmethod
     def add_method(cls, method):
@@ -213,14 +221,6 @@ class QState(ctypes.Structure):
         None
 
         """
-        # for qs in qstates:
-        #     if type(qs) is list or type(qs) is tuple:
-        #         cls.free_all(*qs)
-        #     elif type(qs) is QState:
-        #         qs.free()
-        #     else:
-        #         raise QState_Error_FreeAll()
-
         warnings.warn("No need to call 'free_all' method because free automatically, or you can use class method 'del_all' to free memory explicitly.")
         
     @classmethod
@@ -364,8 +364,9 @@ class QState(ctypes.Structure):
             copy of the original quantum state.
 
         """
-        qstate = qstate_copy(self)
-        return qstate
+        obj = qstate_copy(self)
+        qs = ctypes.cast(obj.value, ctypes.POINTER(self.__class__)).contents
+        return qs
 
     def bloch(self, q=0):
         """
@@ -422,8 +423,6 @@ class QState(ctypes.Structure):
             qs_0 = self.partial(qid=qid)
             qs_1 = qstate.partial(qid=qid)
             inp = qstate_inner_product(qs_0, qs_1)
-            # qs_0.free()
-            # qs_1.free()
         return inp
         
     def tenspro(self, qstate):
@@ -441,8 +440,9 @@ class QState(ctypes.Structure):
             tensor produt of 'self' and 'qstate'.
 
         """
-        qstate_out = qstate_tensor_product(self, qstate)
-        return qstate_out
+        obj = qstate_tensor_product(self, qstate)
+        qs = ctypes.cast(obj.value, ctypes.POINTER(self.__class__)).contents
+        return qs
 
     def fidelity(self, qstate, qid=[]):
         """
@@ -492,9 +492,7 @@ class QState(ctypes.Structure):
             qs = self.clone()
             for i in range(num-1):
                 qs_tmp = qs.tenspro(self)
-                # qs.free()
                 qs = qs_tmp.clone()
-                # qs_tmp.free()
             return qs
         
     def join(self, qs_list):
@@ -515,9 +513,7 @@ class QState(ctypes.Structure):
         qs_out = self.clone()
         for qs in qs_list:
             qs_tmp = qs_out.clone()
-            # qs_out.free()
             qs_out = qs_tmp.tenspro(qs)
-            # qs_tmp.free()
         return qs_out
 
     def evolve(self, observable=None, time=0.0, iter=0):
@@ -602,7 +598,7 @@ class QState(ctypes.Structure):
 
         row = 2**len(qid_0)
         col = 2**len(qid_1)
-        mat = np.zeros((row, col), dtype=np.complex)
+        mat = np.zeros((row, col), dtype=complex)
         for idx,comp in enumerate(vec):
             mat[idx//col][idx%col] = comp
     
@@ -1425,8 +1421,42 @@ class QState(ctypes.Structure):
         return self
     
     # measurement
+
+    def measure(self, qid=None):
+        """
+        one shot measurement in Z-direction.
+
+        Parameters
+        ----------
+        qid : list of int
+            qubit id list to measure.
+
+        Returns
+        -------
+        mval : str
+            measurement value.
+
+        Examples
+        --------
+        >>> qs = QState(qubit_num=2).h(0).cx(0,1)
+        >>> qs.show()
+        >>> print(qs.measure(qid=[0,1]))
+        >>> qs.show()
+        c[00] = +0.7071+0.0000*i : 0.5000 |++++++
+        c[01] = +0.0000+0.0000*i : 0.0000 |
+        c[10] = +0.0000+0.0000*i : 0.0000 |
+        c[11] = +0.7071+0.0000*i : 0.5000 |++++++
+        00
+        c[00] = +1.0000+0.0000*i : 1.0000 |+++++++++++
+        c[01] = +0.0000+0.0000*i : 0.0000 |
+        c[10] = +0.0000+0.0000*i : 0.0000 |
+        c[11] = +0.0000+0.0000*i : 0.0000 |
+
+        """
+        mval = qstate_measure(self, qid=qid, angle=0.0, phase=0.0)
+        return mval
     
-    def m(self, qid=None, shots=DEF_SHOTS, angle=0.0, phase=0.0, tag=None):
+    def m(self, qid=None, shots=DEF_SHOTS, angle=0.0, phase=0.0):
         """
         measurement in any direction (default: Z-axis).
 
@@ -1440,8 +1470,6 @@ class QState(ctypes.Structure):
             direction of measurement (angle with Z-axis).
         phase : float, default 0.0
             direction of measurement (phase around Z-axis).
-        tag : str
-            tag of measurement data.
 
         Returns
         -------
@@ -1463,11 +1491,11 @@ class QState(ctypes.Structure):
         MData class (MData.py)
 
         """
-        md = qstate_measure(self, MDATA_TABLE, qid=qid, shots=shots,
-                              angle=angle, phase=phase, tag=tag)
+        md = qstate_measure_stats(self, qid=qid, shots=shots,
+                                  angle=angle, phase=phase)
         return md
         
-    def mx(self, qid=None, shots=DEF_SHOTS, tag=None):
+    def mx(self, qid=None, shots=DEF_SHOTS):
         """
         X-axis measurement.
 
@@ -1477,8 +1505,6 @@ class QState(ctypes.Structure):
             qubit id list to measure.
         shots : int, default 1
             number of measurements.
-        tag : str
-            tag of measurement data.
 
         Returns
         -------
@@ -1490,11 +1516,11 @@ class QState(ctypes.Structure):
         MData class (MData.py)
 
         """
-        md = qstate_measure(self, MDATA_TABLE, qid=qid, shots=shots,
-                              angle=0.5, phase=0.0, tag=tag)
+        md = qstate_measure_stats(self, qid=qid, shots=shots,
+                                  angle=0.5, phase=0.0)
         return md
         
-    def my(self, qid=None, shots=DEF_SHOTS, tag=None):
+    def my(self, qid=None, shots=DEF_SHOTS):
         """
         Y-axis measurement.
 
@@ -1504,8 +1530,6 @@ class QState(ctypes.Structure):
             qubit id list to measure.
         shots : int, default 1
             number of measurements.
-        tag : str
-            tag of measurement data.
 
         Returns
         -------
@@ -1517,11 +1541,11 @@ class QState(ctypes.Structure):
         MData class (MData.py)
 
         """
-        md =  qstate_measure(self, MDATA_TABLE, qid=qid, shots=shots,
-                              angle=0.5, phase=0.5, tag=tag)
+        md =  qstate_measure_stats(self, qid=qid, shots=shots,
+                                   angle=0.5, phase=0.5)
         return md
         
-    def mz(self, qid=None, shots=DEF_SHOTS, tag=None):
+    def mz(self, qid=None, shots=DEF_SHOTS):
         """
         Z-axis measurement.
 
@@ -1531,8 +1555,6 @@ class QState(ctypes.Structure):
             qubit id list to measure.
         shots : int, default 1
             number of measurements.
-        tag : str
-            tag of measurement data.
 
         Returns
         -------
@@ -1544,120 +1566,13 @@ class QState(ctypes.Structure):
         MData class (MData.py)
 
         """
-        md =  qstate_measure(self, MDATA_TABLE, qid=qid, shots=shots,
-                              angle=0.0, phase=0.0, tag=tag)
+        md =  qstate_measure_stats(self, qid=qid, shots=shots,
+                                   angle=0.0, phase=0.0)
         return md
 
-    def mb(self, qid=None, shots=DEF_SHOTS, tag=None):
-        return qstate_measure_bell(self, MDATA_TABLE, qid=qid, shots=shots, tag=tag)
+    def mb(self, qid=None, shots=DEF_SHOTS):
+        return qstate_measure_bell_stats(self, qid=qid, shots=shots)
         
-    def m_value(self, tag=None, angle=0.0, phase=0.0, binary=False):
-        """
-        get measurement value.
-
-        Parameters
-        ----------
-        tag : str
-            tag of measurement data.
-        angle : float, default 0.0
-            direction of measurement (angle with Z-axis).
-        phase : float, default 0.0
-            direction of measurement (phase around Z-axis).
-        binary : bool
-            format of measurement value.
-
-        Returns
-        -------
-        mval : str (binary:True) or int (binary:False)
-            measurement value (if shots>1, last measured value).
-            - binary=True -> '00','01','10','11' for 2 qubits measurement
-            - binary=False -> 0,1,2,3 for 2 qubits measurement
-
-        Examples
-        --------
-        >>> qs = QState(2).h(0).cx(0,1)
-        >>> qs.m(shots=100)
-        >>> print(qs.m_value(binary=True))
-        00
-
-        """
-        if tag is None: tag = DEF_TAG
-        tag_long = repr(self) + '.' + tag
-        mval = MDATA_TABLE[tag_long].measured_value(angle=angle, phase=phase)
-        if binary == True:
-            digits = len(MDATA_TABLE[tag_long].qid)
-            mval = '{:0{digits}b}'.format(mval, digits=digits)
-        return mval
-
-    def m_bit(self, q, tag=None, angle=0.0, phase=0.0, boolean=False):
-        """
-        get measured bit value.
-
-        Parameters
-        ----------
-        q : int
-        tag : str
-            tag of measurement data.
-        angle : float, default 0.0
-            direction of measurement (angle with Z-axis).
-        phase : float, default 0.0
-            direction of measurement (phase around Z-axis).
-        boolean : bool
-            format of measured bit.
-
-        Returns
-        -------
-        mbit : bool (boolean:True) or int (boolean:False)
-            measured bit value (if shots>1, last measured bit value).
-            - boolean=True -> False,True
-            - boolean=False -> 0,1
-
-        Examples
-        --------
-        >>> qs = QState(2).h(0).cx(0,1)
-        >>> qs.m(shots=100)
-        >>> print(qs.m_bit(boolean=True))
-        True
-
-        """
-        if tag is None: tag = DEF_TAG
-        tag_long = repr(self) + '.' + tag
-        mbit =  MDATA_TABLE[tag_long].measured_bit(q, angle=angle, phase=phase)
-        if boolean == True:
-            mbit = bool(mbit)
-        return mbit
-        
-    def m_freq(self, tag=None, angle=0.0, phase=0.0):
-        """
-        get measurement value.
-
-        Parameters
-        ----------
-        tag : str
-            tag of measurement data.
-        angle : float, default 0.0
-            direction of measurement (angle with Z-axis).
-        phase : float, default 0.0
-            direction of measurement (phase around Z-axis).
-
-        Returns
-        -------
-        mfrq : instance of Counter from collection (Python standard library)
-            measurement frequency.
-
-        Examples
-        --------
-        >>> qs = QState(2).h(0).cx(0,1)
-        >>> qs.m(shots=100)
-        >>> print(qs.m_freq())
-        Counter({'11': 53, '00': 47})
-
-        """
-        if tag is None: tag = DEF_TAG
-        tag_long = repr(self) + '.' + tag
-        mfrq = MDATA_TABLE[tag_long].measured_freq(angle=angle, phase=phase)
-        return mfrq
-
     def operate(self, pp=None, ctrl=None):
         """
         operate unitary operator to quantum state.
@@ -1717,7 +1632,6 @@ class QState(ctypes.Structure):
         None
 
         """
-        # qstate_free(self)
         warnings.warn("No need to call 'free' method because free automatically, or you can use 'del' to free memory explicitly.")
 
     def __del__(self):
