@@ -3,14 +3,14 @@ from .QState import QState
 from .Observable import Observable
 from .DensOp import DensOp
 from .Stabilizer import Stabilizer
-from .QComp import QComp
 from .Backend import Backend
 from .QCirc import QCirc
 from .CMem import CMem
 from .PauliProduct import PauliProduct
+from .Result import Result
 from . import config
 from . import error
 from . import util
 
-__all__ = ["QState","Observable","DensOp","Stabilizer","Qcomp","Backend","QCirc","CMem",
-           "UnitaryOperator","PauliProduct","config","error","util"]
+__all__ = ["QState","Observable","DensOp","Stabilizer","Backend","QCirc","CMem",
+           "UnitaryOperator","PauliProduct","Result","config","error","util"]
