@@ -3,6 +3,7 @@ import ctypes
 from ctypes.util import find_library
 import numpy as np
 import pathlib
+from collections import Counter 
 
 from qlazy.config import *
 from qlazy.error import *
@@ -31,9 +32,7 @@ def qstate_init(qubit_num=None, seed=None):
     if ret == FALSE:
         raise QState_Error_Initialize()
 
-    out = ctypes.cast(c_qstate.value, ctypes.POINTER(QState))
-        
-    return out.contents
+    return c_qstate
 
 def qstate_init_with_vector(vector=None, seed=None):
         
@@ -62,9 +61,7 @@ def qstate_init_with_vector(vector=None, seed=None):
     if ret == FALSE:
         raise QState_Error_Initialize()
     
-    out = ctypes.cast(c_qstate.value, ctypes.POINTER(QState))
-        
-    return out.contents
+    return c_qstate
 
 
 def qstate_reset(qs, qid=None):
@@ -137,9 +134,7 @@ def qstate_copy(qs):
         if ret == FALSE:
             raise QState_Error_Clone()
 
-        out = ctypes.cast(c_qstate.value, ctypes.POINTER(QState))
-
-        return out.contents
+        return c_qstate
         
     except Exception:
         raise QState_Error_Clone()
@@ -178,7 +173,6 @@ def qstate_bloch(qs, q=0):
 def qstate_inner_product(qs_0, qs_1):
 
     try:
-            
         real = 0.0
         imag = 0.0
         c_real = ctypes.c_double(real)
@@ -269,9 +263,7 @@ def qstate_tensor_product(qs, qstate):
         if ret == FALSE:
             raise QState_Error_TensorProduct()
 
-        out = ctypes.cast(c_qstate_out.value, ctypes.POINTER(QState))
-
-        return out.contents
+        return c_qstate_out
 
     except Exception:
         raise QState_Error_TensorProduct()
@@ -332,8 +324,6 @@ def qstate_apply_matrix(qs, matrix=None, qid=None):
 
     if matrix is None:
         raise QState_Error_Apply()
-    if (matrix.shape[0] > qs.state_num or matrix.shape[0] > qs.state_num):
-        raise QState_Error_Apply()
         
     if qid is None or qid == []:
         qid = [i for i in range(qs.qubit_num)]
@@ -350,8 +340,11 @@ def qstate_apply_matrix(qs, matrix=None, qid=None):
         col = row
         size = row * col
 
-        # set array of matrix
-        mat_complex = list(matrix.flatten())
+        mat_complex = []
+        for mat_row in matrix:
+            for mat_elm in mat_row:
+                mat_complex.append(mat_elm)
+        
         mat_real = [0.0 for _ in range(size)]
         mat_imag = [0.0 for _ in range(size)]
         for i in range(size):
@@ -403,11 +396,40 @@ def qstate_operate_qgate(qs, kind=None, qid=None,
     if ret == FALSE:
         raise QState_Error_OperateQgate()
 
-        
-def qstate_measure(qs, MDATA_TABLE, qid=None, shots=DEF_SHOTS, angle=0.0, phase=0.0,
-                   tag=None):
+def qstate_measure(qs, qid=None, angle=0.0, phase=0.0):
 
-    # global MDATA_TABLE
+    if qid is None or qid == []:
+        qid = [i for i in range(qs.qubit_num)]
+
+    # operate
+    qubit_num = len(qid)
+    qubit_id = [0 for _ in range(MAX_QUBIT_NUM)]
+    for i in range(len(qid)):
+        qubit_id[i] = qid[i]
+    IntArray = ctypes.c_int * MAX_QUBIT_NUM
+    qid_array = IntArray(*qubit_id)
+
+    mval = 0
+    c_mval = ctypes.c_int(mval)
+
+    lib.qstate_measure.restype = ctypes.c_int
+    lib.qstate_measure.argtypes = [ctypes.POINTER(QState),
+                                   ctypes.c_double, ctypes.c_double,
+                                   ctypes.c_int, IntArray, ctypes.POINTER(ctypes.c_int)]
+    ret = lib.qstate_measure(ctypes.byref(qs),
+                             ctypes.c_double(angle), ctypes.c_double(phase),
+                             ctypes.c_int(qubit_num), qid_array, ctypes.byref(c_mval))
+
+    if ret == FALSE:
+        raise QState_Error_Measure()
+
+    mval = c_mval.value
+    digits = len(qid)
+    mval = '{:0{digits}b}'.format(mval, digits=digits)
+
+    return mval
+
+def qstate_measure_stats(qs, qid=None, shots=DEF_SHOTS, angle=0.0, phase=0.0):
 
     if qid is None or qid == []:
         qid = [i for i in range(qs.qubit_num)]
@@ -426,14 +448,14 @@ def qstate_measure(qs, MDATA_TABLE, qid=None, shots=DEF_SHOTS, angle=0.0, phase=
     mdata = None
     c_mdata = ctypes.c_void_p(mdata)
 
-    lib.qstate_measure.restype = ctypes.c_int
-    lib.qstate_measure.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
-                                   ctypes.c_double, ctypes.c_double,
-                                   ctypes.c_int, IntArray,
-                                   ctypes.POINTER(ctypes.c_void_p)]
-    ret = lib.qstate_measure(ctypes.byref(qs), ctypes.c_int(shots),
-                             ctypes.c_double(angle), ctypes.c_double(phase),
-                             ctypes.c_int(qubit_num), qid_array, c_mdata)
+    lib.qstate_measure_stats.restype = ctypes.c_int
+    lib.qstate_measure_stats.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
+                                         ctypes.c_double, ctypes.c_double,
+                                         ctypes.c_int, IntArray,
+                                         ctypes.POINTER(ctypes.c_void_p)]
+    ret = lib.qstate_measure_stats(ctypes.byref(qs), ctypes.c_int(shots),
+                                   ctypes.c_double(angle), ctypes.c_double(phase),
+                                   ctypes.c_int(qubit_num), qid_array, c_mdata)
 
     if ret == FALSE:
         raise QState_Error_Measure()
@@ -446,20 +468,13 @@ def qstate_measure(qs, MDATA_TABLE, qid=None, shots=DEF_SHOTS, angle=0.0, phase=
     freq_list = [freq.contents[i] for i in range(state_num)]
     md = MData(freq_list=freq_list, last_state=last_state, qid=qid,
                qubit_num=qubit_num, state_num=state_num, angle=angle, phase=phase,
-               is_bell=False, tag=tag)
+               is_bell=False)
     out.contents.free()
 
-    if tag is None: tag = DEF_TAG
-    tag_long = repr(qs) + '.' + tag
-            
-    MDATA_TABLE[tag_long] = md
-
     return md
 
-def qstate_measure_bell(qs, MDATA_TABLE, qid=None, shots=DEF_SHOTS, tag=None):
+def qstate_measure_bell_stats(qs, qid=None, shots=DEF_SHOTS):
 
-    # global MDATA_TABLE
-        
     if qid is None or qid == []:
         qid = [i for i in range(2)]
             
@@ -478,13 +493,13 @@ def qstate_measure_bell(qs, MDATA_TABLE, qid=None, shots=DEF_SHOTS, tag=None):
     mdata = None
     c_mdata = ctypes.c_void_p(mdata)
 
-    lib.qstate_measure_bell.restype = ctypes.c_int
-    lib.qstate_measure_bell.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
-                                        ctypes.c_int, IntArray,
-                                        ctypes.POINTER(ctypes.c_void_p)]
+    lib.qstate_measure_bell_stats.restype = ctypes.c_int
+    lib.qstate_measure_bell_stats.argtypes = [ctypes.POINTER(QState), ctypes.c_int,
+                                              ctypes.c_int, IntArray,
+                                              ctypes.POINTER(ctypes.c_void_p)]
 
-    ret = lib.qstate_measure_bell(ctypes.byref(qs), ctypes.c_int(shots),
-                                  ctypes.c_int(qubit_num), qid_array, c_mdata)
+    ret = lib.qstate_measure_bell_stats(ctypes.byref(qs), ctypes.c_int(shots),
+                                        ctypes.c_int(qubit_num), qid_array, c_mdata)
 
     if ret == FALSE:
         raise QState_Error_Measure()
@@ -496,51 +511,52 @@ def qstate_measure_bell(qs, MDATA_TABLE, qid=None, shots=DEF_SHOTS, tag=None):
     freq_list = [freq.contents[i] for i in range(state_num)]
     md = MData(freq_list=freq_list, last_state=last_state, qid=qid,
                qubit_num=qubit_num, state_num=state_num, angle=0.0, phase=0.0,
-               is_bell=True, tag=tag)
+               is_bell=True)
     out.contents.free()
 
-    if tag is None: tag = DEF_TAG
-    tag_long = repr(qs) + '.' + tag
-
-    MDATA_TABLE[tag_long] = md
-    
     return md
 
-def qstate_operate_qcirc(qstate, cmem_list, qcirc, shots):
-
-    cmem_num = len(cmem_list)
-    cmem = CMem(cmem_num)
-    
-    mdata = None
-    c_mdata = ctypes.c_void_p(mdata)
+def qstate_operate_qcirc(qstate, cmem, qcirc, shots, cid):
 
     lib.qstate_operate_qcirc.restype = ctypes.c_int
-    lib.qstate_operate_qcirc.argtypes = [ctypes.POINTER(QState), ctypes.POINTER(CMem), ctypes.POINTER(QCirc),
-                                         ctypes.c_int, ctypes.POINTER(ctypes.c_void_p)]
-    ret = lib.qstate_operate_qcirc(ctypes.byref(qstate), ctypes.byref(cmem), ctypes.byref(qcirc),
-                                   ctypes.c_int(shots), c_mdata)
-    
-    if ret == FALSE:
-        raise QState_Error_OperateQcirc()
+    lib.qstate_operate_qcirc.argtypes = [ctypes.POINTER(QState), ctypes.POINTER(CMem), ctypes.POINTER(QCirc)]
+
+    if cmem is not None:
+
+        cmem_num = cmem.cmem_num
+        frequency = Counter()
+        for n in range(shots):
+            
+            if n < shots - 1:
+                qstate_tmp = qstate.clone()
+                cmem_tmp = cmem.clone()
+                ret = lib.qstate_operate_qcirc(ctypes.byref(qstate_tmp), ctypes.byref(cmem_tmp), ctypes.byref(qcirc))
+                bit_array = ctypes.cast(cmem_tmp.bit_array, ctypes.POINTER(ctypes.c_int*cmem_num))
+            else:
+                ret = lib.qstate_operate_qcirc(ctypes.byref(qstate), ctypes.byref(cmem), ctypes.byref(qcirc))
+                bit_array = ctypes.cast(cmem.bit_array, ctypes.POINTER(ctypes.c_int*cmem_num))
+
+            if ret == FALSE:
+                raise QState_Error_OperateQcirc()
+        
+            cmem_list = [bit_array.contents[i] for i in range(cmem_num)]
+            cmem_list_part = [cmem_list[c] for c in cid]
+            mval = "".join(map(str, cmem_list_part))
+
+            frequency[mval] += 1
+
+        return frequency
+
+    else:
+
+        c_cmem = ctypes.POINTER(CMem)()
+        ret = lib.qstate_operate_qcirc(ctypes.byref(qstate), c_cmem, ctypes.byref(qcirc))
+
+        if ret == FALSE:
+            raise QState_Error_OperateQcirc()
 
-    if c_mdata.value == None:
         return None
-        
-    out = ctypes.cast(c_mdata.value, ctypes.POINTER(MDataC))
-        
-    qubit_num = out.contents.qubit_num
-    state_num = out.contents.state_num
-    last_state = out.contents.last
-    qid = [out.contents.qubit_id[i] for i in range(qubit_num)]
-    freq = ctypes.cast(out.contents.freq, ctypes.POINTER(ctypes.c_int*state_num))
-    freq_list = [freq.contents[i] for i in range(state_num)]
-    md = MData(freq_list=freq_list, last_state=last_state, qid=qid,
-               qubit_num=qubit_num, state_num=state_num, angle=0.0, phase=0.0,
-               is_bell=False)
-    out.contents.free()
 
-    return md
-    
 def qstate_free(qs):
 
     lib.qstate_free.argtypes = [ctypes.POINTER(QState)]
